/*
 * @(#)CircleSample.java  1.0  14 July 2001
 *
 * Copyright 2001
 * College of Computer Science
 * Northeastern University
 * Boston, MA  02115
 *
 * This software may be used for educational purposes as long as
 * this copyright notice is retained intact at the top of all files.
 *
 * Should this software be modified, the words "Modified from 
 * Original" must be included as a comment below this notice.
 *
 * All publication rights are retained.  This software or its 
 * documentation may not be published in any media either in whole
 * or in part without explicit permission.
 *
 * Contact information:
 *   Richard Rasala    rasala@ccs.neu.edu
 *   Viera Proulx      vkp@ccs.neu.edu
 *   Jeff Raab         jmr@ccs.neu.edu
 * 
 * Telephone:          617-373-2462
 *
 * This software was created with support from Northeastern 
 * University and from NSF grant DUE-9950829.
 */

import edu.neu.ccs.*;
import edu.neu.ccs.gui.*;
import edu.neu.ccs.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.beans.*;
import javax.swing.*;

public class CircleSample extends DisplayPanel
    implements JPTConstants {
    
    // constant: buffer size
    private static final int bufferSize = 400;
    
    // constants: graph choices
    private static final int graphDRAW = 0;
    private static final int graphFILL = 1;
    
    private static final String[] graphChoices =
        {"Draw", "Fill"};
    
    // constants: center offset choices
    private static final int offsetNone = 4;
    
    private static final String[] offsetChoices =
        {"NW", "N", "NE", "W",
         "None",
         "E", "SW", "S", "SE"};
    
    // input text fields
    private TextFieldView radiusTFV =
        new TextFieldView(
            "20",
            "Fix Radius:",
            "Radius Error");
    
    private TextFieldView xTFV =
        new TextFieldView(
            "200",
            "Fix Center X:",
            "Center Error");
    
    private TextFieldView yTFV =
        new TextFieldView(
            "200",
            "Fix Center Y:",
            "Center Error");
    
    // offset direction options
    private OptionsView offsetOptions =
        new OptionsView
            (offsetChoices,
             offsetNone,
             new GridLayout(3, 3));
    
    // color input via ColorView
    private ColorView color =
        new ColorView(Color.red, true);
    
    // graph options
    private OptionsView graphOptions =
        new OptionsView
            (graphChoices,
             graphFILL,
             new GridLayout(1,0));
    
    // overall control panel for the GUI
    private DisplayCollection controls =
        new DisplayCollection();
    
    // actions panel and its actions
    private ActionsPanel actions =
        new ActionsPanel();
    
    private AbstractAction graph =
        new AbstractAction("Graph") {
            public void actionPerformed(ActionEvent evt) { graph(); }
        };
    
    private AbstractAction reset =
        new AbstractAction("Reset") {
            public void actionPerformed(ActionEvent evt) { reset(); }
        };
    
    // square window for painting one or more circles
    private BufferedPanel window =
        new BufferedPanel(bufferSize, bufferSize);


    // main program
    public static void main(String[] args) {
        JPTFrame.createQuickJPTFrame
            ("Circle Sample Deluxe",new CircleSample());
    }
    

    // constructor
    public CircleSample() {
        installNumericFields();
        installOffsetOptions();
        installColorViews();
        installGraphOptions();
        installActionButtons();
        createMainView();
        reset();
    }
    
    private void installNumericFields() {
        // set the preferred width for the numeric text fields
        radiusTFV.setPreferredWidth(150);
        xTFV.setPreferredWidth(150);
        yTFV.setPreferredWidth(150);
        
        // set the suggestions for the numeric text fields
        radiusTFV.getInputProperties().setSuggestion("20");
        xTFV.getInputProperties().setSuggestion("200");
        yTFV.getInputProperties().setSuggestion("200");
        
        // add the numeric text field views
        controls.add(
          new DisplayWrapper(
            new Display(radiusTFV, "Radius:", null)));
        
        controls.add(
          new DisplayWrapper(
            new Display(xTFV, "Center X:", null)));
        
        controls.add(
          new DisplayWrapper(
            new Display(yTFV, "Center Y:", null)));
    }
    
    private void installOffsetOptions() {
        controls.add(
          new DisplayWrapper(
            new Display(offsetOptions, null, "Center Offset")));
    }
    
    private void installColorViews() {
        color.setChooserClickCount(1);
        
        controls.add(
            new DisplayWrapper(
                new Display(color, null, "Color Selection")));
    }

    private void installGraphOptions() {
        controls.add(
          new DisplayWrapper(
            new Display(graphOptions,null,"Graph Options")));
    }

    public void installActionButtons() {
        // build the actions panel
        actions.addAction(graph);
        actions.addAction(reset);
        
        // add the actions panel
        controls.add(actions);
    }
    
    private void createMainView() {
        // layout for the main panel
        setLayout(new BorderLayout());

        // put the controls and graphics window into displays
        // to title these major areas
        Display controlsDisplay =
            new Display(controls, null, "Controls");

        Display windowDisplay =
            new Display(window, null, "Graphics");

        // add the controls and graphics window to main panel
        add(controlsDisplay, BorderLayout.WEST);
        add(windowDisplay, BorderLayout.CENTER);
    }
    
    // actions
    
    // draw circle
    // offset center depending on user choice
    public void graph() {
        Graphics2D G = window.getBufferGraphics();
        
        // get the user geometry data from the views
		
		double radius;
		double x;
		double y;
		
		/*
		// version using demand
        radius = radiusTFV.demandDouble();
        x = xTFV.demandDouble();
        y = yTFV.demandDouble();
        */
		
		// version using request
		try {
			radius = radiusTFV.requestDouble();
			x = xTFV.requestDouble();
			y = yTFV.requestDouble();
		}
		catch (CancelledException ex) {
			return;
		}
		
		
        // define the diameter
        double diameter = 2 * radius;
        
        // offset center
        int offsetChoice = offsetOptions.getSelectedIndex();
        
        int offsetX = (offsetChoice % 3) - 1;
        int offsetY = (offsetChoice / 3) - 1;
        
        x += offsetX * diameter;
        y += offsetY * diameter;
        
        // define upper left corner
        double xcorner  = x - radius;
        double ycorner  = y - radius;
        
        // define circle
        Ellipse2D.Double E
            = new Ellipse2D.Double(
                xcorner, ycorner, diameter, diameter);
        
        // set paint color to user color choice
        G.setPaint(color.getColor());
        
        // draw or fill circle
        switch (graphOptions.getSelectedIndex()) {
            case graphDRAW:
                G.draw(E);
                break;
            
            case graphFILL:
                G.fill(E);
                break;
            
            default:
                break;
        }
        
        // update text fields
        xTFV.setViewState(x + "");
        yTFV.setViewState(y + "");
        
        repaint();
    }
    
    // reset will erase window and reset controls to initial values
    public void reset() {
        window.clearPanel();
        controls.reset();
        
        repaint();
    }
}